Package: ballgown
Maintainer: Jack Fu <jmfu@jhsph.edu>
Authors@R: c(person("Jack", "Fu", role=c("aut"), email="jmfu@jhsph.edu"),
    person(c("Alyssa", "C."), "Frazee", role=c("aut", "cre"),
    email="alyssa.frazee@gmail.com"), person("Leonardo", "Collado-Torres",
    role=c("aut"), email="lcollado@jhu.edu"), person(c("Andrew", "E."),
    "Jaffe", role=c("aut")), person(c("Jeffrey", "T."), "Leek", role=c("aut",
    "ths"), email="jtleek@gmail.com"))
Version: 2.38.0
License: Artistic-2.0
Title: Flexible, isoform-level differential expression analysis
Description: Tools for statistical analysis of assembled transcriptomes,
    including flexible differential expression analysis, visualization of
    transcript structures, and matching of assembled transcripts to annotation.
Depends: R (>= 3.5.0), methods
Imports: GenomicRanges (>= 1.17.25), IRanges (>= 1.99.22), S4Vectors
        (>= 0.9.39), RColorBrewer, splines, sva, limma, rtracklayer (>=
        1.29.25), Biobase (>= 2.25.0), GenomeInfoDb
Suggests: testthat, knitr, markdown
VignetteBuilder: knitr
BugReports: https://github.com/alyssafrazee/ballgown/issues
biocViews: ImmunoOncology, RNASeq, StatisticalMethod, Preprocessing,
        DifferentialExpression
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/ballgown
git_branch: RELEASE_3_20
git_last_commit: ec0ed55
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29
NeedsCompilation: no
Packaged: 2024-10-29 23:32:08 UTC; biocbuild
Author: Jack Fu [aut],
  Alyssa C. Frazee [aut, cre],
  Leonardo Collado-Torres [aut],
  Andrew E. Jaffe [aut],
  Jeffrey T. Leek [aut, ths]
