#' The ballgown package for analysis of transcript assemblies
#'
#' @name ballgown-package
#' @aliases ballgownp
#' @docType package
#' @import GenomicRanges
#' @import IRanges
#' @import S4Vectors
#' @import RColorBrewer
#' @import splines
#' @importFrom sva f.pvalue
#' @importFrom rtracklayer import
#' @importFrom limma lmFit
#' @import methods
#' @description Super awesome transcript-level expression analysis
NULL
