#' @name bg
#' @title Toy ballgown object
#' @description Small ballgown object created with simulated toy data, for
#'   demonstration purposes
#' @docType data
#' @format a \code{ballgown} object: 100 transcripts, 633 exons, 536 introns
#' @author Alyssa Frazee
#' @examples
#' data(bg)
#' bg
#' # ballgown instance with 100 transcripts and 20 samples
NULL